#include "win_afx.h"

extern "C" DWORD pthread_self() {
	return GetCurrentThreadId();
}

extern "C" int gettimeofday(struct timeval* tv, void* tz) {
	// 获取当前系统时间
	FILETIME ft;
	GetSystemTimeAsFileTime(&ft);

	// 将 FILETIME 转换为 64 位整数
	ULARGE_INTEGER ui;
	ui.LowPart = ft.dwLowDateTime;
	ui.HighPart = ft.dwHighDateTime;

	// 将 100 纳秒转换为秒和微秒
	const long long epoch_offset = 116444736000000000LL; // Windows FILETIME 纪元与 Unix 纪元的差
	ui.QuadPart -= epoch_offset;

	if (tv) {
		tv->tv_sec = static_cast<long>(ui.QuadPart / 10000000LL); // 秒
		tv->tv_usec = static_cast<long>(ui.QuadPart % 10000000LL) / 10; // 微秒
	}

	// tz not set

	return 0; // 成功
}

extern "C" int clock_gettime(int clk_id, struct timespec* ts) {
	if (clk_id == 0) { // CLOCK_REALTIME
		FILETIME ft;
		GetSystemTimeAsFileTime(&ft);

		ULARGE_INTEGER ui;
		ui.LowPart = ft.dwLowDateTime;
		ui.HighPart = ft.dwHighDateTime;

		ui.QuadPart -= 116444736000000000ULL; // 从 1601 到 1970 的偏移
		ts->tv_sec = static_cast<long>(ui.QuadPart / 10000000ULL); // 转为秒
		ts->tv_nsec = static_cast<long>(ui.QuadPart % 10000000ULL) * 100; // 转为纳秒
	}
	else if (clk_id == 1) { // CLOCK_MONOTONIC
		LARGE_INTEGER frequency, counter;
		QueryPerformanceFrequency(&frequency);
		QueryPerformanceCounter(&counter);

		ts->tv_sec = static_cast<long>(counter.QuadPart / frequency.QuadPart);
		ts->tv_nsec = static_cast<long>((counter.QuadPart % frequency.QuadPart) * 1000000000ULL / frequency.QuadPart);
	}
	else {
		return -1; // 不支持的时钟 ID
	}
	return 0; // 成功
}

extern "C" int sched_yield() {
	return SwitchToThread();
}

extern "C" void usleep(int useconds) {
	LARGE_INTEGER frequency;
	LARGE_INTEGER start;
	LARGE_INTEGER end;

	QueryPerformanceFrequency(&frequency);
	QueryPerformanceCounter(&start);

	double microsecondsPerTick = 1000000.0 / frequency.QuadPart;

	do {
		QueryPerformanceCounter(&end);
		Sleep(0);
	} while ((end.QuadPart - start.QuadPart) * microsecondsPerTick < useconds);
}

extern "C" int strncasecmp(const char *s1, const char *s2, size_t n) {
	unsigned char c1, c2;
	do
	{
		c1 = tolower(*s1++);
		c2 = tolower(*s2++);
	} while ((--n > 0) && c1 == c2 && c1 != 0);

	return c1 - c2;
}

extern "C" int strcasecmp(const char *s1, const char *s2) {
	unsigned char c1, c2;
	do
	{
		c1 = tolower(*s1++);
		c2 = tolower(*s2++);
	} while (c1 == c2 && c1 != 0);

	return c1 - c2;
}
